'use strict';

export default
  (({ formID, initiator, realm, request,
    checkForUpdate = false, clientKind = null, clientBuildTimestamp = null,
    transitionDuration = 500, expiredCallback = null }) => {
    /* Set an unsecured user ID */
    try {
      if (typeof request === 'string') {
        request = JSON.parse(request)
      }
      const m = document.cookie.match(/smartforms_uui=(\d+)\b/)
      let userID = null
      if (!m || m.length !== 2) {
        userID = Math.ceil(Math.random() * new Date().getTime())
        document.cookie = `smartforms_uui=${userID}`
      } else {
        userID = m[1]
      }
      request['unsecured_user_id'] = userID
    } catch (e) { }

    const targetFrame = document.getElementById(initiator);
    if (!targetFrame) {
      return;
    }

    /* Loading indicator */
    /* Clean up any orphaned loaders */
    const orphanLoaders = targetFrame.parentNode.querySelectorAll('.smartforms-loader')
    for (var i = 0; i < orphanLoaders.length; i++) {
      if (!document.getElementById(orphanLoaders[i].getAttribute('data-initiator'))) {
        targetFrame.parentNode.removeChild(orphanLoaders[i])
      }
    }
    /* Add a loader */
    const loaderColor = '#aaa';
    const loader = document.createElement('div');
    loader.id = `${initiator}-loader`;
    loader.className = 'smartforms-loader'
    loader.innerHTML = `<style>.lds-ring{display:inline-block;position:relative;width:64px;height:64px}.lds-ring div{box-sizing:border-box;display:block;position:absolute;width:51px;height:51px;margin:6px;border:3px solid ${loaderColor};border-radius:50%;animation:lds-ring 1.2s cubic-bezier(.5,0,.5,1) infinite;border-color:${loaderColor} transparent transparent}.lds-ring div:nth-child(1){animation-delay:-.45s}.lds-ring div:nth-child(2){animation-delay:-.3s}.lds-ring div:nth-child(3){animation-delay:-.15s}@keyframes lds-ring{0%{transform:rotate(0)}100%{transform:rotate(360deg)}}</style>` +
      `<div class="lds-ring"><div></div><div></div><div></div><div></div></div>`;
    loader.style.cssText = `position: absolute; top: 50%; left: 50%;`;
    targetFrame.setAttribute('data-initiator', initiator)
    targetFrame.parentNode.appendChild(loader);

    /* After 10 seconds, set error message on the loader */
    setTimeout(() => {
      const loadEl = document.getElementById(`${initiator}-loader`);
      if (loadEl === null) {
        return
      }
      loadEl.innerHTML = `This facility is currently unavailable. Please try again soon.`;
      loadEl.style.cssText = 'position: absolute; top: 50%; width: 100%; text-align: center;';
    }, 10e3);

    /* The form will POST into the iframe */
    formID = formID || `${initiator}-form`
    let form = document.getElementById(formID);
    /* If the form doesn't exist, create it */
    if (!form) {
      form = document.createElement('form');
      form.id = formID;
      form.style.cssText = 'display: none;';
      const reqEl = document.createElement('input');
      reqEl.type = 'hidden';
      reqEl.name = 'smartformRequest';
      form.appendChild(reqEl);
      document.body.appendChild(form);
    }
    form.action = `${realm}/smartform-runtime-portlet/index`;
    form.method = 'POST';
    form.target = initiator;
    form.querySelector("input[type='hidden']").value = JSON.stringify(request);

    /* Height changes are delayed to prevent rapid changes */
    let heightTimeout = null;
    let heightTimeoutInner = null;
    let nextHeight = null;

    /* Attach the postMessage listener. We only need one instance per window,
       so guard the attachment with a global variable. */
    if (typeof window.smartformsAttached === 'undefined') {
      window.smartformsAttached = true;

      window.addEventListener('message', (event) => {
        if (typeof event.data === 'undefined' || typeof event.data.task === 'undefined' ||
          typeof event.data.targetFrame === 'undefined') {
          return
        }

        /* First thing, remove the loader */
        const foundLoader = document.getElementById(`${event.data.targetFrame}-loader`);
        if (foundLoader !== null) {
          foundLoader.parentNode.removeChild(foundLoader);
        }

        const iframe = document.getElementById(event.data.targetFrame);
        if (!iframe) {
          return
        }
        if (iframe.style.opacity == 0) {
          iframe.style.opacity = 1;
        }

        const task = event.data.task;
        const data = event.data;
        /* 
         * Height: the form is re-adjusting its height.
         * 
         *  data.value: Number, new height in
        */
        if (task === 'height') {
          /* Only apply height changes at-most every 100ms */
          nextHeight = (iframe.offsetHeight - iframe.clientHeight) + data.value + 40;
          if (heightTimeout !== null) {
            return
          }
          heightTimeout = setTimeout(() => {
            if (nextHeight >= iframe.offsetHeight) {
              iframe.height = iframe.style.maxHeight = nextHeight + 'px';
            } else {
              iframe.style.maxHeight = nextHeight + 'px';
              if (heightTimeoutInner === null) {
                heightTimeoutInner = setTimeout(() => {
                  iframe.height = nextHeight + 'px';
                  heightTimeoutInner = null;
                }, transitionDuration);
              }
            }
            heightTimeout = null;
          }, 100);
        }
        /*
         * Scroll: the form is requesting that the iframe viewport is scrolled
         *         to a certain location.
         * 
         *   data.value: Number, scroll offset, currently unused.
         */
        else if (task === 'scroll') {
          try {
            if (typeof iframe.scrollIntoView !== 'undefined') {
              iframe.scrollIntoView({
                behavior: 'smooth',
                block: "start"
              });
              return
            }
            const scrollTarget = iframe.getBoundingClientRect().top + window.scrollY + 20;
            document.documentElement.scrollTop = scrollTarget;
          } catch (err) {
            console.log('SmartForms scroll error', err);
          }
        }
        /*
         * URL: the form is requested that the iframe navigates to another URL.
         * 
         *   data.value: String, URL to navigate to
         */
        else if (task === 'url') {
          iframe.src = data.value;
        }
        /*
         * Started: the form is reporting that it has completed loading
         * 
         * We append the 'loaded' class to the iframe. 
         */
        else if (task === 'started') {
          if (iframe.className.indexOf('loaded') === -1) {
            iframe.className += ' loaded';
          }
        }
        /*
         * Expired: the posted token is expired
         *
         * We invoke the callback or reload the page if absent
        */
        else if (task === 'expired') {
          if (typeof expiredCallback === 'function') {
            expiredCallback();
          } else {
            document.location.reload();
          }
        }
      });
    }

    /* Submit the form after we create the event listener, otherwise
       we might miss some events */
    form.submit();

    /* 
     * If requested, perform a client-side check for a newer version of this client.
     * Only nag once per 24 hours.
    */
    if (checkForUpdate === true && clientKind && clientBuildTimestamp &&
      (document.cookie && document.cookie.match(/smartforms_update_nag=/) === null)) {
      document.cookie = 'smartforms_update_nag=seen;max-age=86400';
      try {
        const xhr = new XMLHttpRequest();
        xhr.addEventListener('load', function () {
          const versions = JSON.parse(this.response);
          const clientInfo = versions[clientKind];
          if (!clientInfo) {
            return
          }
          if (new Date(clientInfo.timestamp) <= new Date(clientBuildTimestamp)) {
            return
          }

          /* Show a toast that can be dismissed by a click */
          const toast = document.createElement('div');
          toast.innerHTML = `There is an update available (v.${clientInfo.version}).<br>` +
            `<a href="${clientInfo.release_notes_url}" target="_blank" rel="noopener noreferrer">View the release notes.</a> or ` +
            `<a href="${clientInfo.upgrade_url}" target="_blank" rel="noopener noreferrer">download the update.</a><br/>` +
            `Click to dismiss for 1 week.`;
          toast.style.cssText =
            'position: absolute; top: 60px; right: 20px; max-width: 400px;' +
            'background: white; border: 1px solid green; cursor: pointer;' +
            'font-family: sans-serif; padding: 1rem;';
          toast.onclick = () => toast.parentNode.removeChild(toast);
          document.body.appendChild(toast);
        });
        xhr.open('GET', 'https://extras.repo.smartfor.ms/clients/release.json');
        xhr.send();
      } catch (e) {
        console.log('Failed to check for update', e);
      }
    }
  });
